#include "param.h"

float4   cnt    : register(c0);
sampler  t0     : register(s0);
sampler  t1     : register(s1);
sampler  t2     : register(s2);

float4 ts_main(float3 x:POSITION):COLOR
{
  return (noise(x * 8) * 0.15 + 0.5);
}

float3 rotate_pos(float3 d, float rot) {
  float3x3 mtx = {
     cos(rot), -sin(rot), 0,
     sin(rot),  cos(rot), 0,
    0,       0,           1
  };
  return mul(mtx, d);
}

float map(float3 p) {
  p += sin(cnt.x) * 0.2;
  float d = tex3Dlod(t2, float4(p, 0) / 4);
  float h = pow(d + 0.5, 16);
  //return abs(h) - dot(p.xy * float2(0, -1), p.xy * float2(0, -1));
  return abs(h) - dot(p.xy, p.xy);
}

float4 raycast(int rep, float3 eye, float3 dir) {
  float4 q[5] = {
    float4(-0.68570,  0.64828,  0.04532, -0.82974),
    float4(-0.47256,  0.26486, -0.59222,  0.72482),
    float4( 0.12552, -0.96567,  0.93915,  0.78464),
    float4(-0.58102, -0.75943,  0.34094,  0.02020),
    float4(-0.16101,  0.86682, -0.52985,  0.13193),
  };
  float3 pao = 0;
  float  ao = 0;
  
  
  //1st
  dir *= 0.001;
  float3 dtdir = dir;
  for(int i = 0; 0 < map(eye) && i < rep; i++) {
    eye += dir;
    dir += dtdir;
    dtdir *= 1.005;
  }
  
  //2nd
  for(int j = 0; j < 62; j++) {
    eye += (dir *= 0.75 ) * sign(map(eye));
  }
  
  //ao
  for(int j = 0; j < 5;j++) {
    pao = map(eye + q[j] * 0.01) * 10;
    ao += pao /  sqrt(pao * pao + 1) + 1;
  }
  return float4(eye, ao);
}

//-------------------------------------------------------------
// getnormal
//-------------------------------------------------------------
float3 getnormal(float3 ip) {
  float2 h      = float2(0.0, 0.01);
  return normalize(float3(map(ip + h.yxx), map(ip + h.xyx), map(ip + h.xxy)) - map(ip));
}

//-------------------------------------------------------------
// ps_main
//-------------------------------------------------------------
float4 ps_main(float2 vp : vpos) : color {
  float sync = cnt.x * 2;
  float pulse = pow((1 - frac(sync)), 8);
  float2 uv   = -1 + 2 * vp / (float2(ScreenX, ScreenY));
  if(abs(uv.y) > 0.75) return 0;
  uv.y = -uv.y;
  float3 dir = normalize(float3(uv * float2(1.25, 1), 1));
  dir = rotate_pos(dir.yzx, cnt.x * 0.1);
  dir = rotate_pos(dir.zxy, cnt.x * 0.1);
  float3   eye = float3(0, 0, cnt.x * 0.5);
  float3   ro  = eye;
  ro += (dir * 0.1) * tex3D(t2, uv.xyx * 100) * 7;
  float4 cast = raycast(150, ro, dir);
  float  dist = length(cast.xyz - eye);
  return dist * cast.w * 0.02 * float4(1, 2, 3, dist);
}

float4 ps_main_f(float2 vp : vpos, float2 uv : texcoord0) : color {
  float4 c = tex2D(t0, uv);
  float2 k = float2(0.0005, 0);
  float4 c1 = tex2D(t0, uv +  k.xy);
  float4 c2 = tex2D(t0, uv + -k.xy);
  float4 c3 = tex2D(t0, uv +  k.yx);
  float4 c4 = tex2D(t0, uv + -k.yx);
  c += c1 + c2 + c3 + c4;
  return (c/5);
}
